/* CS144 Simple Daytime Server
   No error checking, forking/threads, etc...
   Author: Ben Nham (nham@cs) */

#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <signal.h>
#include <string.h>
#include <time.h>

#define BACKLOG 20

/* Not used in this example, but can be useful for simulating "try-finally"
   type constructs. See the CS244A coding guidelines page (linked to from the
   assignment webpage) for example usage.

   Here is an example usage of this macro:
   PRINT_ERR(listen(socketfd, BACKLOG) == -1, cleanup, "listen"); */
#define PRINT_ERR(expr, label, prefix) if (expr) { perror(prefix); goto label; }

/* When the client connects, sends the current day and time in the response. */
void do_server (uint16_t port) {
	int sd, clisd;
	struct sockaddr_in srv, cli;
	u_int clisize = sizeof(cli);
	
	memset(&srv, 0, sizeof(srv));
	srv.sin_family = AF_INET;
	srv.sin_port = htons(port);
	srv.sin_addr.s_addr = INADDR_ANY;
	
	sd = socket(AF_INET, SOCK_STREAM, 0);
	bind(sd, (struct sockaddr *)&srv, sizeof(srv));
	listen(sd, BACKLOG);
	
	while ((clisd = accept(sd, (struct sockaddr *)&cli, &clisize)) != -1) {
		time_t curtime = time(NULL);
		char *timestr = ctime(&curtime);
		
		send(clisd, timestr, strlen(timestr), 0);
		
        /* This is the same as a two-way shutdown. */
		close(clisd);
	}
	
	close(sd);
}

int main (int argc, char **argv) {
	int port = 0;

	if (argc > 2 || (argc == 2 && ((port = atoi (argv[1])) <= 0 || port >= 65536))) {
		fprintf (stderr, "Usage: %s [port]\n", argv[0]);
		exit(1);
	}
    
	do_server((uint16_t) port);
}

